"""
Code generators for MCP Development Toolkit
"""

import json
import os
from datetime import datetime
from pathlib import Path
from typing import Any, Dict, Optional

try:
    import jinja2
    import yaml
except ImportError:
    raise ImportError("Code generation requires full dependencies. Install with: pip install mcp-dev-toolkit[full]")


class ServerGenerator:
    """Generate complete MCP server applications"""
    
    SERVER_TEMPLATE = '''#!/usr/bin/env python3
"""
{{server_name}} - Generated MCP Server
Generated on {{generation_date}}
"""

import asyncio
import json
import logging
from typing import Any, Dict, List

try:
    import mcp.types as types
    from mcp.server import Server
    from mcp.server.models import InitializationOptions
    from mcp.server import NotificationOptions
    from mcp.server.stdio import stdio_server
except ImportError:
    raise ImportError("MCP library not found. Install with: pip install mcp")


class {{server_name.title().replace('-', '')}}Server:
    """{{server_name}} MCP Server Implementation"""
    
    def __init__(self):
        self.server = Server("{{server_name}}")
        self.logger = logging.getLogger("{{server_name}}")
        self._register_handlers()
    
    def _register_handlers(self):
        """Register MCP handlers"""
        
        @self.server.list_tools()
        async def list_tools() -> List[types.Tool]:
            """List available tools"""
            return [
                {% for tool_name, tool_info in tools.items() %}
                types.Tool(
                    name="{{tool_name}}",
                    description="{{tool_info.description}}",
                    inputSchema={
                        "type": "object",
                        "properties": {{tool_info.schema|tojson}},
                        "required": {{tool_info.required|tojson}}
                    }
                ),
                {% endfor %}
            ]
        
        @self.server.call_tool()
        async def call_tool(name: str, arguments: dict) -> List[types.TextContent]:
            """Handle tool calls"""
            try:
                {% for tool_name, tool_info in tools.items() %}
                if name == "{{tool_name}}":
                    result = await self.{{tool_name}}(**arguments)
                    return [types.TextContent(
                        type="text",
                        text=json.dumps(result, indent=2, default=str)
                    )]
                {% endfor %}
                
                return [types.TextContent(
                    type="text", 
                    text=f"Unknown tool: {name}"
                )]
                
            except Exception as e:
                self.logger.error(f"Tool execution failed: {e}")
                return [types.TextContent(
                    type="text",
                    text=f"Error: {str(e)}"
                )]
    
    {% for tool_name, tool_info in tools.items() %}
    async def {{tool_name}}(self, **kwargs) -> Dict[str, Any]:
        """{{tool_info.description}}"""
        # TODO: Implement {{tool_name}} logic
        return {
            "tool": "{{tool_name}}",
            "arguments": kwargs,
            "result": "Generated implementation - please customize",
            "timestamp": datetime.now().isoformat()
        }
    
    {% endfor %}
    
    async def run(self):
        """Run the MCP server"""
        async with stdio_server() as (read_stream, write_stream):
            await self.server.run(
                read_stream,
                write_stream,
                InitializationOptions(
                    server_name="{{server_name}}",
                    server_version="1.0.0",
                    capabilities=self.server.get_capabilities(
                        notification_options=NotificationOptions(),
                        experimental_capabilities={},
                    ),
                ),
            )


def main():
    """Main entry point"""
    logging.basicConfig(level=logging.INFO)
    server = {{server_name.title().replace('-', '')}}Server()
    asyncio.run(server.run())


if __name__ == "__main__":
    main()
'''
    
    @classmethod
    def generate(cls, server_name: str, tools: Dict[str, Any], output_dir: str = ".") -> str:
        """Generate complete server from tools"""
        template = jinja2.Template(cls.SERVER_TEMPLATE)
        
        # Prepare tools data for template
        tools_data = {}
        for tool_name, tool_schema in tools.items():
            tools_data[tool_name] = {
                "description": tool_schema.metadata.description,
                "schema": cls._generate_tool_schema(tool_schema),
                "required": [p.name for p in tool_schema.input_parameters if p.required]
            }
        
        # Generate server code
        server_code = template.render(
            server_name=server_name,
            generation_date=datetime.now().isoformat(),
            tools=tools_data
        )
        
        # Create output directory
        output_path = Path(output_dir) / f"{server_name}_server"
        output_path.mkdir(parents=True, exist_ok=True)
        
        # Write server file
        server_file = output_path / f"{server_name}_server.py"
        server_file.write_text(server_code)
        
        # Generate supporting files
        cls._generate_config(server_name, output_path)
        cls._generate_requirements(output_path)
        cls._generate_readme(server_name, output_path)
        
        return str(output_path)
    
    @classmethod
    def _generate_tool_schema(cls, tool_schema) -> Dict[str, Any]:
        """Generate JSON schema for tool parameters"""
        schema = {}
        for param in tool_schema.input_parameters:
            param_schema = {"type": "string"}  # Default type
            if param.param_type == int:
                param_schema["type"] = "integer"
            elif param.param_type == float:
                param_schema["type"] = "number"
            elif param.param_type == bool:
                param_schema["type"] = "boolean"
            elif param.param_type == list:
                param_schema["type"] = "array"
            elif param.param_type == dict:
                param_schema["type"] = "object"
            
            if param.description:
                param_schema["description"] = param.description
            if param.default is not None:
                param_schema["default"] = param.default
            if param.enum_values:
                param_schema["enum"] = param.enum_values
                
            schema[param.name] = param_schema
        
        return schema
    
    @classmethod
    def _generate_config(cls, server_name: str, output_path: Path):
        """Generate server configuration file"""
        config = {
            "name": server_name,
            "version": "1.0.0",
            "description": f"Generated MCP server: {server_name}",
            "author": "MCP CLI",
            "mcp_version": "1.0"
        }
        
        config_file = output_path / "config.yml"
        config_file.write_text(yaml.dump(config, default_flow_style=False))
    
    @classmethod
    def _generate_requirements(cls, output_path: Path):
        """Generate requirements.txt file"""
        requirements = [
            "mcp>=1.0.0",
            "asyncio",
            "typing-extensions"
        ]
        
        req_file = output_path / "requirements.txt"
        req_file.write_text("\n".join(requirements) + "\n")
    
    @classmethod
    def _generate_readme(cls, server_name: str, output_path: Path):
        """Generate README.md file"""
        readme_content = f"""# {server_name} MCP Server

Generated MCP server created with MCP Development SDK.

## Installation

```bash
pip install -r requirements.txt
```

## Usage

```bash
python {server_name}_server.py
```

## Configuration

Edit `config.yml` to customize server settings.

## Development

This server was generated automatically. You can:

1. Modify the tool implementations in `{server_name}_server.py`
2. Add new tools by editing the source code
3. Update configuration in `config.yml`
4. Test the server with MCP clients

## Generated by MCP Development SDK

Created on {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}
"""
        
        readme_file = output_path / "README.md"
        readme_file.write_text(readme_content)


class OpenAPIGenerator:
    """Generate OpenAPI specifications from MCP tools"""
    
    @classmethod
    def generate(cls, tools: Dict[str, Any], server_name: str, version: str) -> Dict[str, Any]:
        """Generate OpenAPI specification"""
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": f"{server_name} API",
                "version": version,
                "description": f"Auto-generated API for {server_name} MCP server"
            },
            "paths": {},
            "components": {
                "schemas": {}
            }
        }
        
        for tool_name, tool_schema in tools.items():
            # Generate path for each tool
            spec["paths"][f"/tools/{tool_name}"] = {
                "post": {
                    "summary": tool_schema.metadata.description,
                    "requestBody": {
                        "required": True,
                        "content": {
                            "application/json": {
                                "schema": cls._generate_request_schema(tool_schema)
                            }
                        }
                    },
                    "responses": {
                        "200": {
                            "description": "Tool execution result",
                            "content": {
                                "application/json": {
                                    "schema": {"type": "object"}
                                }
                            }
                        }
                    }
                }
            }
        
        return spec
    
    @classmethod
    def _generate_request_schema(cls, tool_schema) -> Dict[str, Any]:
        """Generate request schema for tool"""
        return ServerGenerator._generate_tool_schema(tool_schema)


class ClientGenerator:
    """Generate client libraries for MCP servers"""
    
    @classmethod
    def generate_python_client(cls, server_name: str, tools: Dict[str, Any], output_dir: str) -> str:
        """Generate Python client for server"""
        # This would generate a Python client library
        # For now, return a placeholder
        output_path = Path(output_dir) / f"{server_name}_client.py"
        
        client_code = f'''"""
Python client for {server_name} MCP server
Generated on {datetime.now().isoformat()}
"""

import asyncio
import json
from typing import Any, Dict

class {server_name.title().replace('-', '')}Client:
    """Client for {server_name} MCP server"""
    
    def __init__(self):
        # TODO: Implement MCP client connection
        pass
    
    async def connect(self):
        """Connect to MCP server"""
        # TODO: Implement connection logic
        pass
    
    # Tool methods would be generated here
    # {', '.join(tools.keys())}
'''
        
        output_path.write_text(client_code)
        return str(output_path)
