"""
Middleware system for MCP Development Toolkit
"""

import logging
import traceback
from abc import ABC, abstractmethod
from datetime import datetime
from typing import Any, Optional, Type, Dict, List
from enum import Enum

from .schema import SchemaExtractor


class Middleware(ABC):
    """Base class for middleware"""
    
    def __init__(self, priority: int = 100):
        self.priority = priority
    
    @abstractmethod
    async def process_pre(self, tool_schema: 'ToolSchema', args: tuple, kwargs: dict) -> Optional[tuple]:
        """Process before tool execution"""
        pass
    
    @abstractmethod
    async def process_post(self, tool_schema: 'ToolSchema', result: Any) -> Optional[Any]:
        """Process after tool execution"""
        pass
    
    @abstractmethod
    async def process_error(self, tool_schema: 'ToolSchema', error: Exception, args: tuple, kwargs: dict) -> Optional[Any]:
        """Process when error occurs"""
        pass


class LoggingMiddleware(Middleware):
    """Built-in logging middleware"""
    
    def __init__(self, priority: int = 50):
        super().__init__(priority)
        self.logger = logging.getLogger("mcp_sdk")
    
    async def process_pre(self, tool_schema: 'ToolSchema', args: tuple, kwargs: dict) -> Optional[tuple]:
        self.logger.info(f"Executing tool: {tool_schema.metadata.name}")
        self.logger.debug(f"Args: {args}, Kwargs: {kwargs}")
        return None
    
    async def process_post(self, tool_schema: 'ToolSchema', result: Any) -> Optional[Any]:
        self.logger.info(f"Tool {tool_schema.metadata.name} completed successfully")
        return None
    
    async def process_error(self, tool_schema: 'ToolSchema', error: Exception, args: tuple, kwargs: dict) -> Optional[Any]:
        self.logger.error(f"Tool {tool_schema.metadata.name} failed: {error}")
        self.logger.debug(traceback.format_exc())
        return None


class ValidationMiddleware(Middleware):
    """Input validation middleware"""
    
    def __init__(self, priority: int = 10):
        super().__init__(priority)
    
    async def process_pre(self, tool_schema: 'ToolSchema', args: tuple, kwargs: dict) -> Optional[tuple]:
        # Validate inputs against schema and convert to keyword-only
        validated_kwargs = self._validate_inputs(tool_schema, args, kwargs)
        # Return empty args and all parameters as kwargs to avoid "multiple values" error
        return (), validated_kwargs
    
    async def process_post(self, tool_schema: 'ToolSchema', result: Any) -> Optional[Any]:
        return None
    
    async def process_error(self, tool_schema: 'ToolSchema', error: Exception, args: tuple, kwargs: dict) -> Optional[Any]:
        return None
    
    def _validate_inputs(self, tool_schema: 'ToolSchema', args: tuple, kwargs: dict) -> dict:
        """Validate input parameters - supports both positional and keyword arguments"""
        
        # Create a combined parameter mapping
        combined_params = {}
        
        # Map positional arguments to parameter names
        for i, arg_value in enumerate(args):
            if i < len(tool_schema.input_parameters):
                param = tool_schema.input_parameters[i]
                combined_params[param.name] = arg_value
            else:
                raise ValueError(f"Too many positional arguments. Expected {len(tool_schema.input_parameters)}, got {len(args)}")
        
        # Add keyword arguments (they override positional if same parameter)
        combined_params.update(kwargs)
        
        # Validate all required parameters are present
        for param in tool_schema.input_parameters:
            if param.required and param.name not in combined_params:
                raise ValueError(f"Missing required parameter: {param.name}")
            
            # Handle basic type conversions and validation
            if param.name in combined_params:
                value = combined_params[param.name]
                try:
                    if param.param_type == int:
                        combined_params[param.name] = int(value)
                    elif param.param_type == float:
                        combined_params[param.name] = float(value)
                    elif param.param_type == str:
                        combined_params[param.name] = str(value)
                    elif param.param_type == bool:
                        combined_params[param.name] = bool(value)
                except (ValueError, TypeError):
                    raise ValueError(f"Parameter {param.name} must be of type {param.param_type.__name__}")
                
                # Enum validation
                if param.enum_values and value not in param.enum_values:
                    raise ValueError(f"Parameter {param.name} must be one of: {param.enum_values}")
        
        # Return the validated and converted parameters
        return combined_params


class PerformanceMiddleware(Middleware):
    """Performance monitoring middleware"""
    
    def __init__(self, priority: int = 200):
        super().__init__(priority)
        self.start_time = None
    
    async def process_pre(self, tool_schema: 'ToolSchema', args: tuple, kwargs: dict) -> Optional[tuple]:
        self.start_time = datetime.now()
        return None
    
    async def process_post(self, tool_schema: 'ToolSchema', result: Any) -> Optional[Any]:
        if self.start_time:
            duration = (datetime.now() - self.start_time).total_seconds()
            logging.getLogger("mcp_sdk.performance").info(
                f"Tool {tool_schema.metadata.name} executed in {duration:.3f}s"
            )
        return None
    
    async def process_error(self, tool_schema: 'ToolSchema', error: Exception, args: tuple, kwargs: dict) -> Optional[Any]:
        return None
