"""
Schema extraction utilities for MCP Development Toolkit
"""

import inspect
from enum import Enum
from typing import Any, Callable, Dict, List, Optional, Type, get_type_hints, TYPE_CHECKING

if TYPE_CHECKING:
    from .core import InputParameter


class SchemaExtractor:
    """Extracts schemas from Python function signatures"""
    
    TYPE_MAPPING = {
        str: "string",
        int: "integer", 
        float: "number",
        bool: "boolean",
        list: "array",
        dict: "object",
        List: "array",
        Dict: "object",
        Any: "any"
    }
    
    @classmethod
    def extract_input_parameters(cls, func: Callable) -> List['InputParameter']:
        """Extract input parameters from function signature"""
        from .core import InputParameter
        
        parameters = []
        sig = inspect.signature(func)
        type_hints = get_type_hints(func)
        
        for param_name, param in sig.parameters.items():
            # Skip 'self' and 'cls' parameters
            if param_name in ('self', 'cls'):
                continue
                
            param_type = type_hints.get(param_name, str)
            required = param.default == inspect.Parameter.empty
            default = None if required else param.default
            
            input_param = InputParameter(
                name=param_name,
                param_type=param_type,
                required=required,
                default=default,
                description=f"Parameter: {param_name}"
            )
            
            parameters.append(input_param)
        
        return parameters
    
    @classmethod
    def _extract_enum_values(cls, param_type: Type) -> Optional[List[Any]]:
        """Extract enum values from type annotation"""
        # Handle typing.Literal
        if hasattr(param_type, '__origin__') and hasattr(param_type, '__args__'):
            if str(param_type.__origin__) == 'typing.Literal':
                return list(param_type.__args__)
        
        # Handle Enum classes
        if inspect.isclass(param_type) and issubclass(param_type, Enum):
            return [e.value for e in param_type]
        
        return None
    
    @classmethod
    def generate_json_schema(cls, parameters: List['InputParameter']) -> Dict[str, Any]:
        """Generate JSON schema from parameters"""
        schema = {
            "type": "object",
            "properties": {},
            "required": []
        }
        
        for param in parameters:
            param_schema = {
                "type": cls._get_json_type(param.param_type),
                "description": param.description
            }
            
            if param.enum_values:
                param_schema["enum"] = param.enum_values
            
            if param.default is not None:
                param_schema["default"] = param.default
            
            schema["properties"][param.name] = param_schema
            
            if param.required:
                schema["required"].append(param.name)
        
        return schema
    
    @classmethod
    def _get_json_type(cls, param_type: Type) -> str:
        """Convert Python type to JSON schema type"""
        return cls.TYPE_MAPPING.get(param_type, "string")
    
    @classmethod
    def validate_function_signature(cls, func: Callable) -> Dict[str, Any]:
        """Validate function signature for MCP compatibility"""
        issues = []
        
        try:
            sig = inspect.signature(func)
            type_hints = get_type_hints(func)
            
            # Check if function has return type annotation
            if sig.return_annotation == inspect.Signature.empty:
                issues.append("Missing return type annotation")
            
            # Check parameters
            for param_name, param in sig.parameters.items():
                if param_name in ('self', 'cls'):
                    continue
                    
                if param.annotation == inspect.Parameter.empty and param_name not in type_hints:
                    issues.append(f"Parameter '{param_name}' missing type annotation")
            
            return {
                "valid": len(issues) == 0,
                "issues": issues
            }
            
        except Exception as e:
            return {
                "valid": False,
                "issues": [f"Signature validation error: {str(e)}"]
            }
