#!/usr/bin/env python3
"""
Complete test version of the servers module to check basic functionality
"""

import os
import json
import asyncio
from typing import Dict, List, Optional, Any, Union
from dataclasses import dataclass, asdict
from pathlib import Path
import tempfile
import shutil

from .core import MCPDevelopment, create_sdk, ToolType


@dataclass
class ServerInfo:
    """Information about an available server."""
    name: str
    description: str
    category: str
    version: str
    author: str
    tags: List[str]
    dependencies: List[str]
    config_schema: Dict[str, Any]
    tools_count: int
    github_url: Optional[str] = None
    popularity_score: float = 0.0
    license: str = "MIT"
    created_at: Optional[str] = None
    updated_at: Optional[str] = None


class ServerRegistry:
    """Registry for managing available MCP servers"""
    
    def __init__(self):
        self.servers: Dict[str, ServerInfo] = {}
        self._load_builtin_servers()
    
    def _load_builtin_servers(self):
        """Load built-in server definitions"""
        builtin_servers = [
            ServerInfo(
                name="memory-server",
                description="Personal memory and knowledge management server",
                category="utilities", 
                version="1.0.0",
                author="MCP Team",
                tags=["memory", "knowledge", "notes"],
                dependencies=["sqlite3"],
                config_schema={
                    "database_path": {"type": "string", "default": "./memory.db"},
                    "auto_backup": {"type": "boolean", "default": True}
                },
                tools_count=5,
                popularity_score=95.0
            ),
            ServerInfo(
                name="filesystem-server", 
                description="File system access and management tools",
                category="system",
                version="1.2.0",
                author="MCP Team",
                tags=["filesystem", "files", "directories"],
                dependencies=[],
                config_schema={
                    "allowed_paths": {"type": "array", "default": ["./"]},
                    "max_file_size": {"type": "integer", "default": 10485760}
                },
                tools_count=8,
                popularity_score=88.0
            )
        ]
        
        for server in builtin_servers:
            self.servers[server.name] = server
    
    def register_server(self, server_info: ServerInfo):
        """Register a new server"""
        self.servers[server_info.name] = server_info
    
    def get_server(self, name: str) -> Optional[ServerInfo]:
        """Get server information by name"""
        return self.servers.get(name)
    
    def list_servers(self, category: Optional[str] = None, tags: Optional[List[str]] = None) -> List[ServerInfo]:
        """List available servers with optional filtering"""
        servers = list(self.servers.values())
        
        if category:
            servers = [s for s in servers if s.category == category]
        
        if tags:
            servers = [s for s in servers if any(tag in s.tags for tag in tags)]
        
        return sorted(servers, key=lambda x: x.popularity_score, reverse=True)
    
    def search_servers(self, query: str) -> List[ServerInfo]:
        """Search servers by name, description, or tags"""
        query = query.lower()
        matches = []
        
        for server in self.servers.values():
            if (query in server.name.lower() or 
                query in server.description.lower() or
                any(query in tag.lower() for tag in server.tags)):
                matches.append(server)
        
        return sorted(matches, key=lambda x: x.popularity_score, reverse=True)
    
    def get_categories(self) -> List[str]:
        """Get all available categories"""
        categories = set(server.category for server in self.servers.values())
        return sorted(list(categories))


class ServerDeployer:
    """Deploy MCP servers to various environments"""
    
    def __init__(self):
        self.deployment_history: List[Dict[str, Any]] = []
    
    async def deploy_server(self, server_name: str, output_dir: str, config: Optional[Dict[str, Any]] = None) -> str:
        """Deploy a server to the specified directory"""
        registry = ServerRegistry()
        server_info = registry.get_server(server_name)
        
        if not server_info:
            raise ValueError(f"Server '{server_name}' not found in registry")
        
        # Create deployment directory
        deploy_path = Path(output_dir) / server_name
        deploy_path.mkdir(parents=True, exist_ok=True)
        
        # Generate server implementation
        await self._generate_server_implementation(server_info, deploy_path, config or {})
        
        # Track deployment
        deployment_record = {
            "server_name": server_name,
            "path": str(deploy_path),
            "created": asyncio.get_event_loop().time(),
            "config": config,
            "has_config": bool(config)
        }
        self.deployment_history.append(deployment_record)
        
        return str(deploy_path)
    
    async def _generate_server_implementation(self, server_info: ServerInfo, deploy_path: Path, config: Dict[str, Any]):
        """Generate actual server implementation"""
        # This would generate the actual server code based on server_info
        # For now, create a basic implementation
        
        server_code = f'''#!/usr/bin/env python3
"""
{server_info.name} - MCP Server
{server_info.description}

Generated from server registry
Author: {server_info.author}
Version: {server_info.version}
"""

import asyncio
import logging
from mcp_dev_toolkit import create_sdk, ToolType

# Create SDK instance
sdk = create_sdk("{server_info.name}", "{server_info.version}")

# Configuration
CONFIG = {config!r}

@sdk.tool(
    name="info",
    description="Get server information",
    tool_type=ToolType.FUNCTION,
    tags=["info", "meta"]
)
async def get_server_info() -> dict:
    """Get information about this server"""
    return {{
        "name": "{server_info.name}",
        "description": "{server_info.description}",
        "version": "{server_info.version}",
        "author": "{server_info.author}",
        "category": "{server_info.category}",
        "tags": {server_info.tags!r},
        "tools_count": {server_info.tools_count},
        "config": CONFIG
    }}

# TODO: Add actual tool implementations based on server type

def main():
    """Main entry point"""
    logging.basicConfig(level=logging.INFO)
    app = sdk.create_mcp_app()
    asyncio.run(sdk.run_server(app))

if __name__ == "__main__":
    main()
'''
        
        # Write server file
        server_file = deploy_path / f"{server_info.name}.py"
        server_file.write_text(server_code)
        
        # Write requirements
        requirements = ["mcp-dev-toolkit>=1.0.0"] + server_info.dependencies
        req_file = deploy_path / "requirements.txt"
        req_file.write_text("\n".join(requirements) + "\n")
        
        # Write config file
        config_file = deploy_path / "config.json"
        config_file.write_text(json.dumps(config, indent=2))
        
        # Write README
        readme_content = f"""# {server_info.name}

{server_info.description}

**Author:** {server_info.author}  
**Version:** {server_info.version}  
**Category:** {server_info.category}  
**Tags:** {', '.join(server_info.tags)}

## Installation

```bash
pip install -r requirements.txt
```

## Usage

```bash
python {server_info.name}.py
```

## Configuration

Edit `config.json` to customize server settings.

## Tools

This server provides {server_info.tools_count} tools for {server_info.category} operations.

Generated by MCP Development SDK
"""
        readme_file = deploy_path / "README.md"
        readme_file.write_text(readme_content)



# Global registry instance
_registry = ServerRegistry()
_deployer = ServerDeployer()


# Convenience functions for CLI and external use
def list_available_servers(category: Optional[str] = None, tags: Optional[List[str]] = None) -> List[ServerInfo]:
    """List available servers"""
    return _registry.list_servers(category=category, tags=tags)


def search_servers(query: str) -> List[ServerInfo]:
    """Search for servers by query"""
    return _registry.search_servers(query)


def get_server_info(server_name: str) -> Optional[ServerInfo]:
    """Get information about a specific server"""
    return _registry.get_server(server_name)


async def deploy_server(server_name: str, output_dir: str, config: Optional[Dict[str, Any]] = None) -> str:
    """Deploy a server to the specified directory"""
    return await _deployer.deploy_server(server_name, output_dir, config)


def deploy_server_sync(server_name: str, output_dir: str, config: Optional[Dict[str, Any]] = None) -> str:
    """Synchronous version of deploy_server"""
    return asyncio.run(deploy_server(server_name, output_dir, config))


def get_server_categories() -> List[str]:
    """Get all available server categories"""
    return _registry.get_categories()


def list_deployments(deployments_dir: str = "./deployments") -> List[Dict[str, Any]]:
    """List deployed servers in a directory"""
    deployments_path = Path(deployments_dir)
    deployments = []
    
    if not deployments_path.exists():
        return deployments
    
    for item in deployments_path.iterdir():
        if item.is_dir():
            # Check if it looks like a deployment
            server_files = list(item.glob("*.py"))
            if server_files:
                config_file = item / "config.json"
                has_config = config_file.exists()
                
                # Get creation time
                created = item.stat().st_ctime
                
                deployments.append({
                    "name": item.name,
                    "path": str(item),
                    "created": created,
                    "has_config": has_config,
                    "server_files": [f.name for f in server_files]
                })
    
    return sorted(deployments, key=lambda x: x["created"], reverse=True)


def register_server(server_info: ServerInfo):
    """Register a new server in the global registry"""
    _registry.register_server(server_info)


def get_deployment_history() -> List[Dict[str, Any]]:
    """Get the deployment history"""
    return _deployer.deployment_history.copy()


class ServerManager:
    """High-level server management interface"""
    
    def __init__(self):
        self.registry = _registry
        self.deployer = _deployer
    
    def list_all(self) -> List[ServerInfo]:
        """List all available servers"""
        return self.registry.list_servers()
    
    def find_by_category(self, category: str) -> List[ServerInfo]:
        """Find servers by category"""
        return self.registry.list_servers(category=category)
    
    def find_by_tags(self, tags: List[str]) -> List[ServerInfo]:
        """Find servers by tags"""
        return self.registry.list_servers(tags=tags)
    
    async def quick_deploy(self, server_name: str, target_dir: str = "./servers") -> str:
        """Quickly deploy a server with default settings"""
        return await self.deployer.deploy_server(server_name, target_dir)
    
    def get_popular_servers(self, limit: int = 10) -> List[ServerInfo]:
        """Get most popular servers"""
        all_servers = self.registry.list_servers()
        return all_servers[:limit]  # Already sorted by popularity
    
    def validate_server_config(self, server_name: str, config: Dict[str, Any]) -> Dict[str, Any]:
        """Validate configuration against server schema"""
        server_info = self.registry.get_server(server_name)
        if not server_info:
            return {"valid": False, "errors": [f"Server '{server_name}' not found"]}
        
        errors = []
        schema = server_info.config_schema
        
        # Basic validation
        for key, value in config.items():
            if key not in schema:
                errors.append(f"Unknown config key: {key}")
        
        for key, spec in schema.items():
            if key not in config and spec.get("required", False):
                errors.append(f"Missing required config key: {key}")
        
        return {
            "valid": len(errors) == 0,
            "errors": errors,
            "server_name": server_name
        }
