#!/usr/bin/env python3
"""
MCP Development Toolkit - Template System

This module provides comprehensive template management for creating production-ready
MCP servers and applications. Templates include both standalone Python implementations
and integration utilities for existing JavaScript servers.

Key Features:
- Template registry and management
- Multi-language server support (Python primary, JavaScript integration)
- Production-ready application templates
- Configuration management and validation
- Deployment orchestration

Template Categories:
- Core MCP Servers (Python)
- JavaScript Integration Bridges
- Full Application Stacks
- Deployment Configurations
"""

from .registry import TemplateRegistry, TemplateInfo
from .manager import TemplateManager
from .base import BaseTemplate

# Template generator imports
from .generator import (
    generate_template,
    scaffold_template_files,
    create_template_structure,
    validate_template_config,
    TemplateValidator
)

# Core template functions
def list_available_templates(category: str = None) -> list:
    """List all available templates"""
    registry = TemplateRegistry()
    return registry.list_templates(category=category)


def create_from_template(template_name: str, output_path: str, config: dict = None) -> str:
    """Create a project from template"""
    manager = TemplateManager()
    return manager.create_from_template(template_name, output_path, config or {})


def get_template_info(template_name: str) -> TemplateInfo:
    """Get information about a specific template"""
    registry = TemplateRegistry()
    return registry.get_template(template_name)


def list_template_categories() -> list:
    """List all available template categories"""
    registry = TemplateRegistry()
    return registry.get_categories()


def validate_template_config(template_name: str, config: dict) -> dict:
    """Validate configuration for a template"""
    validator = TemplateValidator()
    return validator.validate_config(template_name, config)


# Template generator functions
def create_new_template(name: str, category: str, description: str, output_path: str) -> str:
    """Create a new template"""
    return generate_template(
        name=name,
        category=category, 
        description=description,
        output_path=output_path
    )


def validate_existing_template(template_path: str) -> dict:
    """Validate an existing template"""
    validator = TemplateValidator()
    return validator.validate_template_structure(template_path)


def get_available_template_types() -> list:
    """Get available template types"""
    return [
        "server",
        "application", 
        "integration",
        "utility",
        "bridge"
    ]


def get_available_template_categories() -> list:
    """Get available template categories"""
    return [
        "core",
        "api",
        "integration", 
        "utilities",
        "examples",
        "bridges"
    ]


__all__ = [
    # Core classes
    "TemplateRegistry",
    "TemplateManager", 
    "TemplateInfo",
    "BaseTemplate",
    
    # Generator functions
    "generate_template",
    "scaffold_template_files",
    "create_template_structure",
    "validate_template_config",
    "TemplateValidator",
    
    # Convenience functions
    "list_available_templates",
    "create_from_template",
    "get_template_info", 
    "list_template_categories",
    "create_new_template",
    "validate_existing_template",
    "get_available_template_types",
    "get_available_template_categories",
]
