#!/usr/bin/env python3
"""
Base Template Classes - Foundation for Template System

This module provides base classes and utilities for template creation
and management.
"""

from abc import ABC, abstractmethod
from typing import Dict, Any, List, Optional
from pathlib import Path


class BaseTemplate(ABC):
    """Base class for all templates."""
    
    def __init__(self, name: str, version: str, description: str):
        self.name = name
        self.version = version
        self.description = description
    
    @abstractmethod
    def get_configuration_schema(self) -> Dict[str, Any]:
        """Get the configuration schema for this template."""
        pass
    
    @abstractmethod
    def get_dependencies(self) -> List[str]:
        """Get the list of dependencies for this template."""
        pass
    
    @abstractmethod
    def generate_files(self, output_path: Path, config: Dict[str, Any]) -> List[str]:
        """Generate template files in the output path."""
        pass
    
    def validate_config(self, config: Dict[str, Any]) -> Dict[str, Any]:
        """Validate configuration against schema."""
        schema = self.get_configuration_schema()
        errors = []
        
        # Check required fields
        for key, spec in schema.items():
            if spec.get("required", False) and key not in config:
                errors.append(f"Missing required field: {key}")
        
        # Basic type validation
        for key, value in config.items():
            if key in schema:
                spec = schema[key]
                expected_type = spec.get("type", "string")
                
                if expected_type == "string" and not isinstance(value, str):
                    errors.append(f"Field '{key}' must be a string")
                elif expected_type == "integer" and not isinstance(value, int):
                    errors.append(f"Field '{key}' must be an integer")
                elif expected_type == "boolean" and not isinstance(value, bool):
                    errors.append(f"Field '{key}' must be a boolean")
        
        return {
            "valid": len(errors) == 0,
            "errors": errors
        }
    
    def get_template_info(self) -> Dict[str, Any]:
        """Get template information."""
        return {
            "name": self.name,
            "version": self.version,
            "description": self.description,
            "configuration_schema": self.get_configuration_schema(),
            "dependencies": self.get_dependencies()
        }


class PythonServerTemplate(BaseTemplate):
    """Template for Python MCP servers."""
    
    def __init__(self, name: str = "python-server", version: str = "1.0.0"):
        super().__init__(
            name=name,
            version=version,
            description="Basic Python MCP server template"
        )
    
    def get_configuration_schema(self) -> Dict[str, Any]:
        """Get configuration schema for Python server."""
        return {
            "server_name": {
                "type": "string",
                "required": True,
                "description": "Name of the MCP server"
            },
            "port": {
                "type": "integer",
                "default": 8000,
                "description": "Port number for the server"
            },
            "log_level": {
                "type": "string",
                "default": "INFO",
                "enum": ["DEBUG", "INFO", "WARNING", "ERROR"],
                "description": "Logging level"
            }
        }
    
    def get_dependencies(self) -> List[str]:
        """Get dependencies for Python server."""
        return [
            "mcp-dev-toolkit>=1.0.0",
            "asyncio"
        ]
    
    def generate_files(self, output_path: Path, config: Dict[str, Any]) -> List[str]:
        """Generate Python server files."""
        files_created = []
        
        server_name = config.get("server_name", "my-server")
        
        # Main server file
        server_code = f'''#!/usr/bin/env python3
"""
{server_name} - MCP Server
Generated by Python Server Template
"""

import asyncio
import logging
from mcp_dev_toolkit import create_sdk, ToolType

# Configuration
CONFIG = {config!r}

# Create SDK instance
sdk = create_sdk("{server_name}", "1.0.0")

@sdk.tool(
    name="hello",
    description="Say hello",
    tool_type=ToolType.FUNCTION,
    tags=["greeting"]
)
async def hello(name: str = "World") -> dict:
    """Say hello to someone"""
    return {{
        "message": f"Hello, {{name}}!",
        "server": "{server_name}",
        "config": CONFIG
    }}

def main():
    """Main entry point"""
    logging.basicConfig(level=getattr(logging, CONFIG.get("log_level", "INFO")))
    app = sdk.create_mcp_app()
    asyncio.run(sdk.run_server(app))

if __name__ == "__main__":
    main()
'''
        
        server_file = output_path / f"{server_name}.py"
        server_file.write_text(server_code)
        files_created.append(str(server_file))
        
        # Requirements file
        requirements = self.get_dependencies()
        req_file = output_path / "requirements.txt"
        req_file.write_text("\n".join(requirements) + "\n")
        files_created.append(str(req_file))
        
        # README file
        readme_content = f"""# {server_name}

MCP Server generated from Python Server Template.

## Installation

```bash
pip install -r requirements.txt
```

## Usage

```bash
python {server_name}.py
```

## Configuration

Server configuration: {config}

Generated by MCP Development SDK
"""
        readme_file = output_path / "README.md"
        readme_file.write_text(readme_content)
        files_created.append(str(readme_file))
        
        return files_created


class APIServerTemplate(BaseTemplate):
    """Template for API-based MCP servers."""
    
    def __init__(self, name: str = "api-server", version: str = "1.0.0"):
        super().__init__(
            name=name,
            version=version,
            description="REST API MCP server template"
        )
    
    def get_configuration_schema(self) -> Dict[str, Any]:
        """Get configuration schema for API server."""
        return {
            "api_title": {
                "type": "string",
                "required": True,
                "description": "Title of the API"
            },
            "api_version": {
                "type": "string",
                "default": "1.0.0",
                "description": "Version of the API"
            },
            "host": {
                "type": "string",
                "default": "0.0.0.0",
                "description": "Host to bind the server"
            },
            "port": {
                "type": "integer",
                "default": 8000,
                "description": "Port number for the server"
            }
        }
    
    def get_dependencies(self) -> List[str]:
        """Get dependencies for API server."""
        return [
            "mcp-dev-toolkit>=1.0.0",
            "fastapi>=0.68.0",
            "uvicorn>=0.15.0"
        ]
    
    def generate_files(self, output_path: Path, config: Dict[str, Any]) -> List[str]:
        """Generate API server files."""
        # For now, generate basic files similar to Python server
        # In a full implementation, this would create FastAPI-specific code
        python_template = PythonServerTemplate()
        return python_template.generate_files(output_path, config)


class TemplateFactory:
    """Factory for creating template instances."""
    
    TEMPLATE_CLASSES = {
        "python-server": PythonServerTemplate,
        "api-server": APIServerTemplate,
    }
    
    @classmethod
    def create_template(cls, template_type: str, **kwargs) -> BaseTemplate:
        """Create a template instance by type."""
        if template_type not in cls.TEMPLATE_CLASSES:
            raise ValueError(f"Unknown template type: {template_type}")
        
        template_class = cls.TEMPLATE_CLASSES[template_type]
        return template_class(**kwargs)
    
    @classmethod
    def list_available_types(cls) -> List[str]:
        """List available template types."""
        return list(cls.TEMPLATE_CLASSES.keys())
