#!/usr/bin/env python3
"""
Template Generator - Create new templates for the MCP Development Toolkit

This module provides tools for generating new template structures, including:
- Template directory scaffolding
- Jinja2 template file generation
- Configuration schema creation
- Documentation generation
- Example configurations
- Template validation and testing
"""

from .core import TemplateGeneratorCore
from .scaffolder import TemplateScaffolder
from .validator import TemplateValidator
from .examples import ExampleGenerator

# Main interface functions
def generate_template(name: str, category: str, description: str,
                     template_type: str = "server", output_path: str = ".", 
                     config: dict = None):
    """Generate a new template structure."""
    generator = TemplateGeneratorCore()
    return generator.generate_template(name, category, description, template_type, output_path, config)

def scaffold_template_files(template_path: str, template_info: dict):
    """Create the basic file structure for a template."""
    scaffolder = TemplateScaffolder()
    return scaffolder.create_template_structure(template_path, template_info)

def create_template_structure(template_name: str, output_path: str, 
                            template_config: dict = None):
    """Create complete template directory structure."""
    scaffolder = TemplateScaffolder()
    return scaffolder.scaffold_complete_template(template_name, output_path, template_config or {})

def validate_template_config(template_path: str):
    """Validate template configuration and structure."""
    validator = TemplateValidator()
    return validator.validate_template_structure(template_path)

class TemplateValidator:
    """Validator for template structures and configurations."""
    
    def validate_template_structure(self, template_path: str) -> dict:
        """Validate template directory structure."""
        # Basic validation implementation
        from pathlib import Path
        import json
        
        path = Path(template_path)
        errors = []
        
        if not path.exists():
            return {"valid": False, "errors": ["Template path does not exist"]}
        
        # Check for required files
        required_files = ["template_config.json"]
        for req_file in required_files:
            if not (path / req_file).exists():
                errors.append(f"Missing required file: {req_file}")
        
        return {
            "valid": len(errors) == 0,
            "errors": errors,
            "path": str(path)
        }
    
    def validate_config(self, template_name: str, config: dict) -> dict:
        """Validate configuration for a template."""
        # Basic config validation
        return {
            "valid": True,
            "errors": [],
            "template": template_name
        }


__all__ = [
    "generate_template",
    "scaffold_template_files", 
    "create_template_structure",
    "validate_template_config",
    "TemplateValidator",
    "TemplateGeneratorCore",
    "TemplateScaffolder",
    "ExampleGenerator"
]
