#!/usr/bin/env python3
"""
Template Generator Core - Main template generation logic
"""

from pathlib import Path
from typing import Dict, Any
import json


class TemplateGeneratorCore:
    """Core template generation functionality."""
    
    def generate_template(self, name: str, category: str, description: str,
                         template_type: str = "server", output_path: str = ".", 
                         config: dict = None) -> str:
        """Generate a new template structure."""
        config = config or {}
        
        # Create template directory
        template_dir = Path(output_path) / name
        template_dir.mkdir(parents=True, exist_ok=True)
        
        # Create template config
        template_config = {
            "template": {
                "name": name,
                "version": "1.0.0",
                "description": description,
                "category": category,
                "type": template_type,
                "author": "Generated",
                "language": "python"
            },
            "features": ["basic_server", "tool_registration"],
            "dependencies": ["mcp-dev-toolkit>=1.0.0"],
            "configuration_schema": {
                "server_name": {"type": "string", "required": True},
                "port": {"type": "integer", "default": 8000}
            }
        }
        
        # Write template config
        config_file = template_dir / "template_config.json"
        config_file.write_text(json.dumps(template_config, indent=2))
        
        # Create basic template files
        main_template = template_dir / "main.py.j2"
        main_template.write_text('''#!/usr/bin/env python3
"""
{{config.server_name}} - MCP Server
Generated from template: {{template_info.name}}
"""

import asyncio
import logging
from mcp_dev_toolkit import create_sdk, ToolType

# Create SDK instance
sdk = create_sdk("{{config.server_name}}", "1.0.0")

@sdk.tool(
    name="hello",
    description="Say hello",
    tool_type=ToolType.FUNCTION,
    tags=["greeting"]
)
async def hello(name: str = "World") -> dict:
    """Say hello to someone"""
    return {
        "message": f"Hello, {name}!",
        "server": "{{config.server_name}}",
        "template": "{{template_info.name}}"
    }

def main():
    """Main entry point"""
    logging.basicConfig(level=logging.INFO)
    app = sdk.create_mcp_app()
    asyncio.run(sdk.run_server(app))

if __name__ == "__main__":
    main()
''')
        
        # Create README template
        readme_template = template_dir / "README.md.j2"
        readme_template.write_text('''# {{config.server_name}}

{{template_info.description}}

## Installation

```bash
pip install -r requirements.txt
```

## Usage

```bash
python {{config.server_name}}.py
```

Generated by MCP Development SDK
Template: {{template_info.name}}
''')
        
        # Create requirements template
        req_template = template_dir / "requirements.txt.j2"
        req_template.write_text('''mcp-dev-toolkit>=1.0.0
asyncio
''')
        
        return str(template_dir)
