#!/usr/bin/env python3
"""
Template Validator - Validate template structures and configurations
"""

from pathlib import Path
from typing import Dict, Any
import json


class TemplateValidator:
    """Validate template structures and configurations."""
    
    def validate_template_structure(self, template_path: str) -> dict:
        """Validate template directory structure."""
        path = Path(template_path)
        errors = []
        
        if not path.exists():
            return {"valid": False, "errors": ["Template path does not exist"]}
        
        # Check for required files
        required_files = ["template_config.json"]
        for req_file in required_files:
            if not (path / req_file).exists():
                errors.append(f"Missing required file: {req_file}")
        
        # Validate template config if it exists
        config_file = path / "template_config.json"
        if config_file.exists():
            try:
                with open(config_file) as f:
                    config = json.load(f)
                
                # Validate config structure
                required_keys = ["template"]
                for key in required_keys:
                    if key not in config:
                        errors.append(f"Missing required config key: {key}")
                        
            except json.JSONDecodeError as e:
                errors.append(f"Invalid JSON in template_config.json: {e}")
        
        return {
            "valid": len(errors) == 0,
            "errors": errors,
            "path": str(path)
        }
    
    def validate_config(self, template_name: str, config: dict) -> dict:
        """Validate configuration for a template."""
        return {
            "valid": True,
            "errors": [],
            "template": template_name
        }
