#!/usr/bin/env python3
"""
Template Manager - Template Instantiation and Project Creation

This module handles the creation of new projects from templates,
including file generation, configuration, and setup.
"""

import os
import shutil
import json
from pathlib import Path
from typing import Dict, Any, Optional, List
from jinja2 import Environment, FileSystemLoader, Template
import yaml

from .registry import TemplateRegistry, TemplateInfo
from .base import BaseTemplate


class TemplateManager:
    """Manager for creating projects from templates."""
    
    def __init__(self, registry: Optional[TemplateRegistry] = None):
        self.registry = registry or TemplateRegistry()
        self.templates_base_path = self._get_templates_base_path()
    
    def _get_templates_base_path(self) -> str:
        """Get the base path for template files."""
        return os.path.join(os.path.dirname(__file__), 'applications')
    
    def create_from_template(self, template_name: str, output_path: str, config: Dict[str, Any]) -> str:
        """Create a new project from template"""
        template_info = self.registry.get_template(template_name)
        if not template_info:
            raise ValueError(f"Template '{template_name}' not found")
        
        # Validate configuration
        validation_result = self.validate_config(template_info, config)
        if not validation_result["valid"]:
            raise ValueError(f"Invalid configuration: {validation_result['errors']}")
        
        # Create output directory
        output_dir = Path(output_path)
        output_dir.mkdir(parents=True, exist_ok=True)
        
        # Get template path
        template_path = Path(self.templates_base_path) / template_name
        
        if template_path.exists():
            # Use file-based template
            return self._create_from_file_template(template_info, template_path, output_dir, config)
        else:
            # Use programmatic template generation
            return self._create_from_programmatic_template(template_info, output_dir, config)
    
    def _create_from_file_template(self, template_info: TemplateInfo, template_path: Path, output_dir: Path, config: Dict[str, Any]) -> str:
        """Create project from file-based template"""
        # Setup Jinja2 environment
        env = Environment(loader=FileSystemLoader(str(template_path)))
        
        # Process each file in template
        for root, dirs, files in os.walk(template_path):
            rel_root = os.path.relpath(root, template_path)
            
            # Create corresponding directory structure
            if rel_root != '.':
                target_dir = output_dir / rel_root
                target_dir.mkdir(parents=True, exist_ok=True)
            
            # Process each file
            for file in files:
                if file.startswith('.') or file.endswith('.pyc'):
                    continue
                    
                source_file = Path(root) / file
                rel_file_path = source_file.relative_to(template_path)
                
                # Determine target path
                target_file = output_dir / rel_file_path
                
                # Process template file
                if file.endswith('.j2'):
                    # Remove .j2 extension for target
                    target_file = target_file.with_suffix('')
                    
                    # Render template
                    template = env.get_template(str(rel_file_path))
                    rendered_content = template.render(**config, template_info=template_info)
                    
                    target_file.write_text(rendered_content)
                else:
                    # Copy file as-is
                    shutil.copy2(source_file, target_file)
        
        return str(output_dir)
    
    def _create_from_programmatic_template(self, template_info: TemplateInfo, output_dir: Path, config: Dict[str, Any]) -> str:
        """Create project using programmatic template generation"""
        # Create basic project structure based on template type
        if template_info.category == "core":
            return self._create_basic_server_template(template_info, output_dir, config)
        elif template_info.category == "api":
            return self._create_api_server_template(template_info, output_dir, config)
        elif template_info.category == "integration":
            return self._create_integration_template(template_info, output_dir, config)
        else:
            return self._create_generic_template(template_info, output_dir, config)
    
    def _create_basic_server_template(self, template_info: TemplateInfo, output_dir: Path, config: Dict[str, Any]) -> str:
        """Create basic MCP server template"""
        server_name = config.get("server_name", "my-server")
        
        # Main server file
        server_code = f'''#!/usr/bin/env python3
"""
{server_name} - MCP Server
Generated from template: {template_info.name}
"""

import asyncio
import logging
from mcp_dev_toolkit import create_sdk, ToolType

# Configuration
CONFIG = {config!r}

# Create SDK instance
sdk = create_sdk("{server_name}", "1.0.0")

@sdk.tool(
    name="hello",
    description="Say hello with server information",
    tool_type=ToolType.FUNCTION,
    tags=["greeting", "info"]
)
async def hello(name: str = "World") -> dict:
    """Say hello and provide server information"""
    return {{
        "message": f"Hello, {{name}}! This is {{CONFIG.get('server_name', 'the server')}}",
        "server": "{server_name}",
        "template": "{template_info.name}",
        "version": "1.0.0",
        "config": CONFIG
    }}

@sdk.tool(
    name="get_config",
    description="Get current server configuration",
    tool_type=ToolType.FUNCTION,
    tags=["config", "info"]
)
async def get_config() -> dict:
    """Get server configuration"""
    return {{
        "server_name": "{server_name}",
        "template": "{template_info.name}",
        "config": CONFIG
    }}

def main():
    """Main entry point"""
    logging.basicConfig(level=getattr(logging, CONFIG.get("log_level", "INFO")))
    app = sdk.create_mcp_app()
    asyncio.run(sdk.run_server(app))

if __name__ == "__main__":
    main()
'''
        
        # Write server file
        server_file = output_dir / f"{server_name}.py"
        server_file.write_text(server_code)
        
        # Create requirements.txt
        requirements = ["mcp-dev-toolkit>=1.0.0"] + template_info.dependencies
        req_file = output_dir / "requirements.txt"
        req_file.write_text("\n".join(requirements) + "\n")
        
        # Create configuration file
        config_file = output_dir / "config.json"
        config_file.write_text(json.dumps(config, indent=2))
        
        # Create README
        readme_content = f"""# {server_name}

MCP Server generated from template: {template_info.name}

## Description
{template_info.description}

## Installation

```bash
pip install -r requirements.txt
```

## Usage

```bash
python {server_name}.py
```

## Configuration

Edit `config.json` to customize server settings.

## Features

{chr(10).join(f"- {feature}" for feature in template_info.features)}

## Generated by MCP Development SDK

Template: {template_info.name} v{template_info.version}
Author: {template_info.author}
"""
        readme_file = output_dir / "README.md"
        readme_file.write_text(readme_content)
        
        return str(output_dir)
    
    def _create_api_server_template(self, template_info: TemplateInfo, output_dir: Path, config: Dict[str, Any]) -> str:
        """Create API server template with FastAPI"""
        # Simplified API server template
        return self._create_basic_server_template(template_info, output_dir, config)
    
    def _create_integration_template(self, template_info: TemplateInfo, output_dir: Path, config: Dict[str, Any]) -> str:
        """Create integration bridge template"""
        # Simplified integration template  
        return self._create_basic_server_template(template_info, output_dir, config)
    
    def _create_generic_template(self, template_info: TemplateInfo, output_dir: Path, config: Dict[str, Any]) -> str:
        """Create generic template"""
        return self._create_basic_server_template(template_info, output_dir, config)
    
    def validate_config(self, template_info: TemplateInfo, config: Dict[str, Any]) -> Dict[str, Any]:
        """Validate configuration against template schema"""
        errors = []
        schema = template_info.configuration_schema
        
        # Check required fields
        for key, spec in schema.items():
            if spec.get("required", False) and key not in config:
                errors.append(f"Missing required field: {key}")
        
        # Check field types and values
        for key, value in config.items():
            if key in schema:
                spec = schema[key]
                expected_type = spec.get("type", "string")
                
                # Basic type validation
                if expected_type == "string" and not isinstance(value, str):
                    errors.append(f"Field '{key}' must be a string")
                elif expected_type == "integer" and not isinstance(value, int):
                    errors.append(f"Field '{key}' must be an integer")
                elif expected_type == "boolean" and not isinstance(value, bool):
                    errors.append(f"Field '{key}' must be a boolean")
                
                # Enum validation
                if "enum" in spec and value not in spec["enum"]:
                    errors.append(f"Field '{key}' must be one of: {spec['enum']}")
        
        return {
            "valid": len(errors) == 0,
            "errors": errors,
            "template": template_info.name
        }
    
    def list_available_templates(self, category: Optional[str] = None) -> List[TemplateInfo]:
        """List available templates"""
        return self.registry.list_templates(category=category)
    
    def get_template_info(self, template_name: str) -> Optional[TemplateInfo]:
        """Get template information"""
        return self.registry.get_template(template_name)
    
    def preview_template_files(self, template_name: str, config: Dict[str, Any]) -> List[str]:
        """Preview what files would be created from template"""
        template_info = self.registry.get_template(template_name)
        if not template_info:
            return []
        
        # This is a simplified preview - in reality would show actual file structure
        files = [
            f"{config.get('server_name', 'server')}.py",
            "requirements.txt",
            "config.json",
            "README.md"
        ]
        
        if template_info.category == "api":
            files.extend(["openapi.json", "Dockerfile"])
        
        return files
    
    def validate_template_structure(self, template_path: str) -> Dict[str, Any]:
        """Validate template directory structure"""
        path = Path(template_path)
        
        if not path.exists():
            return {"valid": False, "errors": ["Template path does not exist"]}
        
        errors = []
        
        # Check for required files
        required_files = ["template_config.json"]
        for req_file in required_files:
            if not (path / req_file).exists():
                errors.append(f"Missing required file: {req_file}")
        
        # Validate template config if it exists
        config_file = path / "template_config.json"
        if config_file.exists():
            try:
                with open(config_file) as f:
                    config = json.load(f)
                
                # Validate config structure
                required_keys = ["template", "category", "description"]
                for key in required_keys:
                    if key not in config:
                        errors.append(f"Missing required config key: {key}")
                        
            except json.JSONDecodeError as e:
                errors.append(f"Invalid JSON in template_config.json: {e}")
        
        return {
            "valid": len(errors) == 0,
            "errors": errors,
            "path": str(path)
        }
