#!/usr/bin/env python3
"""
Template Registry - Template Discovery and Management

This module manages the registry of available templates, their metadata,
versioning, and compatibility information.
"""

import json
import os
from typing import Dict, List, Optional, Any
from dataclasses import dataclass, asdict
from pathlib import Path
import yaml


@dataclass
class TemplateInfo:
    """Template information and metadata."""
    name: str
    version: str
    description: str
    category: str
    author: str
    language: str
    sdk_compatibility: str
    tags: List[str]
    dependencies: List[str]
    features: List[str]
    configuration_schema: Dict[str, Any]
    examples: List[Dict[str, Any]]
    created_at: Optional[str] = None
    updated_at: Optional[str] = None
    github_url: Optional[str] = None
    documentation_url: Optional[str] = None
    license: str = "MIT"


class TemplateRegistry:
    """Registry for managing available templates"""
    
    def __init__(self):
        self.templates: Dict[str, TemplateInfo] = {}
        self._load_builtin_templates()
    
    def _load_builtin_templates(self):
        """Load built-in template definitions"""
        builtin_templates = [
            TemplateInfo(
                name="basic-server",
                version="1.0.0",
                description="Basic MCP server with essential functionality",
                category="core",
                author="MCP Team",
                language="python",
                sdk_compatibility=">=1.0.0",
                tags=["basic", "starter", "server"],
                dependencies=["mcp-dev-toolkit"],
                features=["tool_registration", "basic_handlers", "logging"],
                configuration_schema={
                    "server_name": {"type": "string", "required": True},
                    "port": {"type": "integer", "default": 8000},
                    "log_level": {"type": "string", "default": "INFO", "enum": ["DEBUG", "INFO", "WARNING", "ERROR"]}
                },
                examples=[
                    {"name": "hello_world", "description": "Simple greeting tool"}
                ]
            ),
            TemplateInfo(
                name="api-server",
                version="1.2.0", 
                description="REST API server with OpenAPI documentation",
                category="api",
                author="MCP Team",
                language="python",
                sdk_compatibility=">=1.0.0",
                tags=["api", "rest", "openapi", "web"],
                dependencies=["mcp-dev-toolkit", "fastapi", "uvicorn"],
                features=["rest_api", "openapi_docs", "request_validation", "async_handlers"],
                configuration_schema={
                    "api_title": {"type": "string", "required": True},
                    "api_version": {"type": "string", "default": "1.0.0"},
                    "host": {"type": "string", "default": "0.0.0.0"},
                    "port": {"type": "integer", "default": 8000},
                    "enable_docs": {"type": "boolean", "default": True}
                },
                examples=[
                    {"name": "user_management", "description": "User CRUD operations"},
                    {"name": "data_processing", "description": "Data transformation tools"}
                ]
            ),
            TemplateInfo(
                name="integration-bridge",
                version="1.0.0",
                description="Bridge for integrating with existing JavaScript servers",
                category="integration", 
                author="MCP Team",
                language="javascript",
                sdk_compatibility=">=1.0.0",
                tags=["integration", "bridge", "javascript", "nodejs"],
                dependencies=["@modelcontextprotocol/sdk"],
                features=["js_interop", "python_bridge", "event_handling"],
                configuration_schema={
                    "python_server_url": {"type": "string", "required": True},
                    "nodejs_port": {"type": "integer", "default": 3000},
                    "timeout": {"type": "integer", "default": 30000}
                },
                examples=[
                    {"name": "python_tool_bridge", "description": "Bridge Python tools to JS"}
                ]
            )
        ]
        
        for template in builtin_templates:
            self.templates[template.name] = template
    
    def register_template(self, template_info: TemplateInfo):
        """Register a new template"""
        self.templates[template_info.name] = template_info
    
    def get_template(self, name: str) -> Optional[TemplateInfo]:
        """Get template information by name"""
        return self.templates.get(name)
    
    def list_templates(self, category: Optional[str] = None, language: Optional[str] = None) -> List[TemplateInfo]:
        """List available templates with optional filtering"""
        templates = list(self.templates.values())
        
        if category:
            templates = [t for t in templates if t.category == category]
        
        if language:
            templates = [t for t in templates if t.language == language]
        
        return sorted(templates, key=lambda x: (x.category, x.name))
    
    def search_templates(self, query: str) -> List[TemplateInfo]:
        """Search templates by name, description, or tags"""
        query = query.lower()
        matches = []
        
        for template in self.templates.values():
            if (query in template.name.lower() or 
                query in template.description.lower() or
                any(query in tag.lower() for tag in template.tags)):
                matches.append(template)
        
        return sorted(matches, key=lambda x: x.name)
    
    def get_categories(self) -> List[str]:
        """Get all available categories"""
        categories = set(template.category for template in self.templates.values())
        return sorted(list(categories))
    
    def get_languages(self) -> List[str]:
        """Get all available languages"""
        languages = set(template.language for template in self.templates.values())
        return sorted(list(languages))
    
    def get_templates_by_category(self, category: str) -> List[TemplateInfo]:
        """Get all templates in a specific category"""
        return [t for t in self.templates.values() if t.category == category]
    
    def get_templates_by_language(self, language: str) -> List[TemplateInfo]:
        """Get all templates for a specific language"""
        return [t for t in self.templates.values() if t.language == language]
    
    def validate_template_compatibility(self, template_name: str, sdk_version: str) -> bool:
        """Check if template is compatible with SDK version"""
        template = self.get_template(template_name)
        if not template:
            return False
        
        # Simple version comparison (would be more sophisticated in production)
        try:
            required = template.sdk_compatibility.replace(">=", "").strip()
            return sdk_version >= required
        except:
            return True  # Assume compatible if can't parse
    
    def export_registry(self, output_path: str):
        """Export registry to JSON file"""
        registry_data = {
            "templates": {
                name: asdict(template) for name, template in self.templates.items()
            },
            "metadata": {
                "total_templates": len(self.templates),
                "categories": self.get_categories(),
                "languages": self.get_languages()
            }
        }
        
        with open(output_path, 'w') as f:
            json.dump(registry_data, f, indent=2, default=str)
    
    def import_registry(self, registry_path: str):
        """Import templates from JSON file"""
        with open(registry_path, 'r') as f:
            registry_data = json.load(f)
        
        for name, template_dict in registry_data.get('templates', {}).items():
            template = TemplateInfo(**template_dict)
            self.register_template(template)
    
    def get_template_stats(self) -> Dict[str, Any]:
        """Get statistics about the template registry"""
        templates = list(self.templates.values())
        
        stats = {
            "total_templates": len(templates),
            "by_category": {},
            "by_language": {},
            "by_author": {},
            "total_features": set(),
            "total_dependencies": set()
        }
        
        for template in templates:
            # Count by category
            stats["by_category"][template.category] = stats["by_category"].get(template.category, 0) + 1
            
            # Count by language
            stats["by_language"][template.language] = stats["by_language"].get(template.language, 0) + 1
            
            # Count by author
            stats["by_author"][template.author] = stats["by_author"].get(template.author, 0) + 1
            
            # Collect features and dependencies
            stats["total_features"].update(template.features)
            stats["total_dependencies"].update(template.dependencies)
        
        # Convert sets to lists for JSON serialization
        stats["total_features"] = sorted(list(stats["total_features"]))
        stats["total_dependencies"] = sorted(list(stats["total_dependencies"]))
        
        return stats
